/* --COPYRIGHT--,BSD_EX
 * Copyright (c) 2013, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *******************************************************************************/
//******************************************************************************
//   MSP432P401 Demo
//    DCO=3MHz -> MCLK, SMCLK. REFO -> ACLK, BCLK, BCLK -> WDT, RTC
//	  (ADC14: ADC14SC(SW), MODCLK)
//    waked up by P6 and WDT, (ADC_Temp, RTC)
//    (LPM0: ADC active), LPM3: P6, WDT, RTC
//
//   MSP432P401x
//		-----------------
//		|        -> P5	|--->+ connect P6
//      |        <- P6	|<---+ PIO to wake-up
//		|	  <-WDT,ADC |
//
//******************************************************************************
#include "msp.h"
#include <stdio.h>
#include <stdint.h>
#include "EventLib.h"

#define mclk_MHz	3

uint16_t	Event_num, ev_irqn, ev_flag;
uint8_t		p5bit = 0, p6ifg;

uint16_t	error_interrupt = 0;

int main(void) {

    WDT_A->CTL = WDT_A_CTL_PW | WDT_A_CTL_HOLD;               // Stop WDT

    P1->DIR |= BIT0;							// P1.0 output
    P1->OUT &= ~BIT0;               			// P1.0 LED off
    P5->OUT = 0; P5->DIR = 0xFF;				// P5.0~7 output

    // Configure P6 for interrupt, pull-down
    P6->OUT = 0;	                			// set Pull-down on P6
    P6->REN = 0xFF;
    P6->IES = 0;								// P6.7 Low->High edge
    P6->IFG = 0;	  	                      	// Clear P6IFG
    P6->IE |= 0xFF;                           	// P6.0~7 interrupt enabled
    
    /* REFO -> ACLK/BCLK, DCO=3MHz -> MCLK, SMCLK */
    CS->KEY = CS_KEY_VAL ;                      // Unlock CS module
    CS->CTL0 = CS_CTL0_DCORSEL_2;
    CS->CTL1 |= CS_CTL1_SELA_2 | CS_CTL1_SELB;
    CS->KEY = 0;                                // Lock CS module

    // WDT:interval timer = 1s, clock source is BCLK(REFO)
    WDT_A->CTL = WDT_A_CTL_PW | WDT_A_CTL_TMSEL | WDT_A_CTL_CNTCL |
    				WDT_A_CTL_SSEL_3 | WDT_A_CTL_IS_4;

    // Wake up by WFE
    //SCB->SCR = SCB_SCR_SEVONPEND_Msk;

    // Wake up after ISR, set Deep-Sleep
    SCB->SCR |= (SCB_SCR_SLEEPDEEP_Msk);
    // set LPM3, force entry LPMx
    PCM->CTL0 = PCM_CTL0_KEY_VAL | PCM_CTL0_LPMR__LPM3;
    PCM->CTL1 = PCM_CTL0_KEY_VAL | PCM_CTL1_FORCE_LPM_ENTRY;

    // enable NVIC interrupt of WDT interval, P6
    NVIC->ISER[0] = 1 << ((WDT_A_IRQn) & 31);
    if (PORT6_IRQn >= 32)   NVIC->ISER[1] = 1 << ((PORT6_IRQn-32) & 31);
    else					NVIC->ISER[0] = 1 << ((PORT6_IRQn) & 31);
    __enable_interrupt();

  while(1)
  {
	Event_num = Event_Read( );
	ev_irqn = (Event_num & 0x00FF);
	ev_flag = (Event_num >> 8);
//	printf ("IRQ %d flag %x\n", ev_irqn, ev_flag );
    switch ( ev_irqn )
    {
    case	WDT_A_IRQn:
		// interrupt to P6
		if(p5bit ==0) p5bit = 3;			// generate multi-interrupt
		P5->OUT = p5bit;
		p5bit = p5bit << 2;
		break;
    case	PORT6_IRQn:
    	printf ("IRQ %d flag %x\n", ev_irqn, ev_flag );
    	p6ifg = ev_flag;
    	// dummy: multi IFG bits
    	if((p6ifg & 1)) { }
    	else if((p6ifg & 2)) { }
    	//P1->OUT ^= BIT0;                          // Toggle P1.0 LED
        break;
    case	Event_false:
    	P1->OUT |= BIT0;                          // Toggle P1.0 LED
    	__sleep();
    	__no_operation();                   // For debugger
    	P1->OUT &= ~BIT0;
    	break;
    default:
    	error_interrupt++;
    	break;
    }
  }
}

/* Watchdog Timer interrupt service routine */
void WDT_A_IRQHandler(void)
{
	Event_Send( WDT_A_IRQn, 1 );
}

/* Port 6 interrupt service routine */
void PORT6_IRQHandler(void)
{
	uint8_t ev_flag =0;
	ev_flag = P6->IFG;				// can send multi IFG bits
	Event_Send( PORT6_IRQn, ev_flag );
	ev_flag = P6->IV;				// clear P6IV
/*
	switch(P6->IV)
	{
	case	DIO_PORT_IV__IFG0:
		{	ev_flag = BIT0; break;	}
	case	DIO_PORT_IV__IFG1:
		{	ev_flag = BIT1; break;	}
	case	DIO_PORT_IV__IFG2:
		{	ev_flag = BIT2; break;	}
	case	DIO_PORT_IV__IFG3:
		{	ev_flag = BIT3; break;	}
	case	DIO_PORT_IV__IFG4:
		{	ev_flag = BIT4; break;	}
	case	DIO_PORT_IV__IFG5:
		{	ev_flag = BIT5; break;	}
	case	DIO_PORT_IV__IFG6:
		{	ev_flag = BIT6; break;	}
	case	DIO_PORT_IV__IFG7:
		{	ev_flag = BIT7; break;	}

	default:
		break;
	}
	Event_Send( PORT6_IRQn, ev_flag );
*/
}
