/* --COPYRIGHT--,BSD_EX
 * Copyright (c) 2013, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *******************************************************************************/
/*
 * Event library
 *
 *
 */
#include "EventLib.h"
static	uint16_t	Event_bits[4];
static	uint8_t		EventPool[4][16];
static	uint64_t	EventLpm0 = 0;

void Event_Send( uint8_t ev_irqn, uint8_t ev_flag )
{
	uint16_t	i;
	if( ev_irqn < 16 ) 			{ i=0; }
	else if( ev_irqn < 32 )		{ i=1; ev_irqn = ev_irqn-16; }
	else if( ev_irqn < 48 )		{ i=2; ev_irqn = ev_irqn-32; }
	else if( ev_irqn < 64 )		{ i=3; ev_irqn = ev_irqn-48; }
	else	return;							// error

	Event_bits[i] |= ( 1 << (ev_irqn) );
	EventPool[i][ev_irqn] |= ev_flag;
	return;
}

uint16_t Event_Read( )
{
	uint16_t	i, k, wk_bits;
	uint8_t		wk_flag;

	// check event
	if( Event_bits[0] ) 	 { k=0; }
	else if( Event_bits[1] ) { k=1; }
	else if( Event_bits[2] ) { k=2; }
	else if( Event_bits[3] ) { k=3; }
	else	return Event_false16;				// No event
	// get ev_flag
	wk_bits = Event_bits[k];
	if( wk_bits & 0x000F ) 		{ i = 0; }
	else if( wk_bits & 0x00F0 ) { i = 4; wk_bits = (wk_bits >> 4);}
	else if( wk_bits & 0x0F00 ) { i = 8; wk_bits = (wk_bits >> 8);}
	else 						{ i = 12; wk_bits = (wk_bits >> 12); }

	while(1)
	{
		if( wk_bits & 1 )
		{
			Event_bits[k] &= ~( 1 << i );		// Event_bits off
			break;
		}
		wk_bits = wk_bits >> 1;
		i++;
	}

	wk_flag = EventPool[k][i];					// read ev_flag
	EventPool[k][i] &= ~wk_flag;				// clear saved flag
	//return ev_irqn(L) and ev_flag(H)
	return  ((wk_flag << 8) | ( (k << 4) + i));
}

void 	Event_UseLpm0( uint8_t ev_irqn )
{
	EventLpm0 |= (1 << ((ev_irqn) & 63));
}

void 	Event_ClearLpm0( uint8_t ev_irqn )
{
	EventLpm0 &= ~(1 << ((ev_irqn) &63));
}

uint16_t Event_SeeLpm0()
{
	if( EventLpm0 != 0) return 1;		// return true
	return 0;
}

