/* --COPYRIGHT--,BSD_EX
 * Copyright (c) 2013, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/
//******************************************************************************
//   MSP432P401 Demo
//    Vcore1, DCO=24MHz -> MCLK, 1/2SMCLK, Flash wait=0
//    REFO -> ACLK -> TimerA, REFO -> BCLK -> WDT, RTC
//	  ADC14: ADC14SC(SW), MODCLK, measure 1/2AVCC by REF 2.5V
//    waked up by P6 and WDT, RTC,
//    LPM0: ADC, TimerA,  LPM3: P6 interrupt, WDT, RTC-event by every minute
//    LED on while LPM3 waiting
//
//   MSP432P401x
//		-----------------
//		|        -> P5	|--->+ connect P6
//      |        <- P6	|<---+ PIO to wake-up
//		|	  <-WDT,ADC |
//		|	       P4.2 |58--> timings of functions (g)
//		|	       P4.3 |59--> timings of APP  (p)
//		|          p1.0 |4 --> indicate sleeping LPM3 (y)
//		|          p1.1 |5 --> indicate sleeping LPM0 (b)
//
//******************************************************************************
#include "msp.h"
#include <stdio.h>
#include <stdint.h>
#include "EventLib.h"

void	RTC_start();
void	ADC_start();
void	ADC_stop();
uint16_t	ADC_data;
void 	TimerA0_start (uint16_t ccr0 );
void 	TimerA0_stop ();

uint16_t	Event_num, ev_irqn, ev_flag;
uint8_t		p5bit = 0, p6ifg;
uint16_t	error_interrupt = 0;

int main(void) {

    WDT_A->CTL = WDT_A_CTL_PW | WDT_A_CTL_HOLD;               // Stop WDT

    /* Step 1: Transition to VCORE Level 1: AM0_LDO --> AM1_LDO */
    PCM->CTL0 = PCM_CTL0_KEY_VAL | PCM_CTL0_AMR_1;
    while ((PCM->CTL1 & PCM_CTL1_PMR_BUSY));
    if ((PCM->CTL0 & PCM_CTL0_CPM_MASK) != PCM_CTL0_CPM_1)    while(1);
    /* Step2 : configure Flash wait, No wait */

    /* Step3 : Configure DCO to 24MHz -> MCLK=24MHz, 1/2->SMCLK=12MHz
     * 			REFO -> ACLK, -> BCLK									 */
    CS->KEY = CS_KEY_VAL ;                  // Unlock CS module
    //CS->CTL0 = 0;                           // Reset tuning parameters
    CS->CTL0 = CS_CTL0_DCORSEL_4;           // Set DCO to 24MHz
    CS->CTL1 = CS_CTL1_SELA_2 | CS_CTL1_SELS_3 | CS_CTL1_SELM_3 |
    			CS_CTL1_SELB | CS_CTL1_DIVS_1 | CS_CTL1_DIVHS_1;
    CS->KEY = 0;                             // Lock CS module

    /* configure Port  */
    P1->DIR |= BIT0 | BIT1;							// P1.0,1 output
    P1->OUT &= ~(BIT0 | BIT1);             			// P1.0 LED off
    // function timings-> P4.2, MCLK -> P4.3
    P4->DIR |= BIT2 | BIT3;
    P5->OUT = 0; P5->DIR = 0xFF;				// P5.0~7 output
    // Configure P6 for interrupt, pull-down
    P6->OUT = 0;	                			// set Pull-down on P6
    P6->REN = 0xFF;
    P6->IES = 0;								// P6.7 Low->High edge
    P6->IFG = 0;	  	                      	// Clear P6IFG
    P6->IE |= 0xFF;                           	// P6.0~7 interrupt enabled

    // Wake up by pending event
    //SCB->SCR = SCB_SCR_SEVONPEND_Msk;

    // enable NVIC interrupt of WDT interval, P6, RTC, ADC14, TimerA0
    NVIC->ISER[0] = 1 << ((WDT_A_IRQn) & 31);
    if (PORT6_IRQn >= 32)   NVIC->ISER[1] = 1 << ((PORT6_IRQn-32) & 31);
    else					NVIC->ISER[0] = 1 << ((PORT6_IRQn) & 31);
    NVIC->ISER[0] = 1 << ((ADC14_IRQn) & 31);
    NVIC->ISER[0] = 1 << ((RTC_C_IRQn) & 31);
    NVIC->ISER[0] = 1 << ((TA0_0_IRQn) & 31);
    __enable_interrupt();

    // WDT:interval timer = 1s, clock source is BCLK(REFO)
    WDT_A->CTL = WDT_A_CTL_PW | WDT_A_CTL_TMSEL | WDT_A_CTL_CNTCL |
    				WDT_A_CTL_SSEL_3 | WDT_A_CTL_IS_4;
    // start RTC, setting to wake-up by every minute event
    RTC_start();

  while(1)
  {
	P4->OUT |= BIT2;						// output timings of vent_Read
	Event_num = Event_Read( );
	ev_irqn = (Event_num & 0x00FF);
	ev_flag = (Event_num >> 8);
	P4->OUT &= ~BIT2;
	//printf ("IRQ %d flag %x\n", ev_irqn, ev_flag );
    switch ( ev_irqn )
    {
    case	WDT_A_IRQn:
    	// request TimerA0 to start
    	P4->OUT |= BIT2;					// output timings of vent_Read
    	Event_Send( TA0_0_IRQn, 1 );
    	P4->OUT &= ~BIT2;
		// interrupt to P6
		if(p5bit ==0) p5bit = 3;			// generate multi-interrupt
		P5->OUT = p5bit;
		p5bit = p5bit << 2;
		break;
    case	TA0_0_IRQn:
    	if( ev_flag ==1 )
    	{	// start TimerA0
    		Event_UseLpm0( TA0_0_IRQn );
    		P4->OUT |= BIT3;				// APP timing on
    		TimerA0_start ( 32768/10 );		// 100ms
    		P4->OUT &= ~BIT3;				// APP timing off
    	}
    	else
    	{	// event from TimeA0 ISR, stop TimerA0
    		TimerA0_stop();
    		Event_ClearLpm0( TA0_0_IRQn );
    	}
    	break;
    case	PORT6_IRQn:
    	//printf ("IRQ %d flag %x\n", ev_irqn, ev_flag );
		P4->OUT |= BIT3;				// APP timing on
    	p6ifg = ev_flag;
    	/* dummy: check IFG bits
    	if((p6ifg & BIT0)) { }
    	else if((p6ifg & BIT1)) { }
    	else if((p6ifg & BIT2)) { }
    	*/
		P4->OUT &= ~BIT3;				// APP timing off
        break;
    case	RTC_C_IRQn:						// every minutes
    	// trigger ADC
    	//printf ("Minu %d\n", ev_flag );
    	Event_Send( ADC14_IRQn, 1 );
    	break;
    case	ADC14_IRQn:
    	if( ev_flag ==1 )
    	{
    		Event_UseLpm0( ADC14_IRQn );
    		ADC_start();
    	}
    	else
    	{	// event from ADC ISR
    		/* AVCC=3.3V => 3.3/2/2.5(vref)* 0x4000(full) =about 10813 */
    		ADC_stop();
    		Event_ClearLpm0( ADC14_IRQn );
    		//printf ("AVCC/2 %d\n", ADC_data );
    	}
    	break;
    case	Event_false:
    	if( Event_SeeLpm0() )
    	{	// LPM0
    	    // reset Deep-Sleep
    		P1->OUT |= BIT1;					// P1.1 on while LPM0
    	    SCB->SCR &= ~(SCB_SCR_SLEEPDEEP_Msk);
    	    // reset LPM3 and force entry
    	    PCM->CTL0 = PCM_CTL0_KEY_VAL;
    	    PCM->CTL1 = PCM_CTL0_KEY_VAL;
    		__sleep();
    		__no_operation();                   // For debugger
    		P1->OUT &= ~BIT1;
    	}
    	else
    	{	// LPM3
    		P1->OUT |= BIT0;					// P1.0 on while LPM3
    	    // set Deep-Sleep
    	    SCB->SCR |= (SCB_SCR_SLEEPDEEP_Msk);
    	    // set LPM3, force entry LPMx
    	    PCM->CTL0 = PCM_CTL0_KEY_VAL | PCM_CTL0_LPMR__LPM3;
    	    PCM->CTL1 = PCM_CTL0_KEY_VAL | PCM_CTL1_FORCE_LPM_ENTRY;
    		__sleep();
    		__no_operation();                   // For debugger
    		P1->OUT &= ~BIT0;
    	}
    	break;
    default:
    	error_interrupt++;
    	break;

    }
  }
}

/* Watchdog Timer interrupt service routine */
void WDT_A_IRQHandler(void)
{
	Event_Send( WDT_A_IRQn, 1 );
}

/* Port 6 interrupt service routine */
void PORT6_IRQHandler(void)
{
	uint8_t ev_flag =0;
	ev_flag = P6->IFG;				// can send multi IFG bits
	Event_Send( PORT6_IRQn, ev_flag );
	ev_flag = P6->IV;				// clear P6IV
/*
	switch(P6->IV)
	{
	case	DIO_PORT_IV__IFG0:
		{	ev_flag = BIT0; break;	}
	case	DIO_PORT_IV__IFG1:
		{	ev_flag = BIT1; break;	}
	case	DIO_PORT_IV__IFG2:
		{	ev_flag = BIT2; break;	}
	case	DIO_PORT_IV__IFG3:
		{	ev_flag = BIT3; break;	}
	case	DIO_PORT_IV__IFG4:
		{	ev_flag = BIT4; break;	}
	case	DIO_PORT_IV__IFG5:
		{	ev_flag = BIT5; break;	}
	case	DIO_PORT_IV__IFG6:
		{	ev_flag = BIT6; break;	}
	case	DIO_PORT_IV__IFG7:
		{	ev_flag = BIT7; break;	}

	default:
		break;
	}
	Event_Send( PORT6_IRQn, ev_flag );
*/
}


void	RTC_start()
{
    /* Configure RTC
     * Unlock RTC key protected registers, RTC enable, binary mode, RTC hold
     * enable RTC time event interrupt, when minute changes */
        RTC_C->CTL0 = RTC_C_KEY | RTC_C_CTL0_TEVIE;
        RTC_C->CTL13 = RTC_C_CTL13_HOLD | RTC_C_CTL13_MODE | RTC_C_CTL13_TEV_0;

        RTC_C->YEAR = 2017;		                   		// Year
        RTC_C->DATE = (1 << RTC_C_DATE_MON_OFS) | 		// Month
                ( 5 | RTC_C_DATE_DAY_OFS);    			// Day
        RTC_C->TIM1 = ( 4 << RTC_C_TIM1_DOW_OFS) | 	// Day of week, THU.
                ( 10 << RTC_C_TIM1_HOUR_OFS);  			// Hour
        RTC_C->TIM0 = ( 1 << RTC_C_TIM0_MIN_OFS) | 	// Minute
                ( 50 << RTC_C_TIM0_SEC_OFS);	   		// Seconds

        // Start RTC calendar mode
        RTC_C->CTL13 = RTC_C->CTL13 & ~(RTC_C_CTL13_HOLD);

        // Lock the RTC registers
        RTC_C->CTL0 = RTC_C->CTL0 & ~(RTC_C_CTL0_KEY_MASK);
}

// RTC interrupt service routine
void RTC_C_IRQHandler(void)
{
	uint8_t	rtc_minu;		// RTC minute
	switch(RTC_C->IV)
    {
	/*case	RTC_C_CTL0_TEVIFG:
		Event_Send( RTC_C_IRQn, RTC_C_CTL0_TEVIFG );
		break;*/
	case	0x06:
		rtc_minu = (uint8_t)(RTC_C->TIM0 >> RTC_C_TIM0_MIN_OFS);
		Event_Send( RTC_C_IRQn, rtc_minu );
		break;
	default:
		Event_Send( RTC_C_IRQn, 0xFF );
		break;
    }
}

void TimerA0_start (uint16_t ccr0 )
{
	// ACLK-> TA0, UP mode, TACCR0 interrupt enabled
	TIMER_A0->CCTL[0] &= ~TIMER_A_CCTLN_CCIFG;
	TIMER_A0->CCTL[0] = TIMER_A_CCTLN_CCIE;
	TIMER_A0->CCR[0] = ccr0;
	TIMER_A0->CTL = TIMER_A_CTL_SSEL__ACLK | TIMER_A_CTL_CLR |
					 TIMER_A_CTL_MC__UP;
}

void TimerA0_stop(void)
{
	TIMER_A0->CTL = TIMER_A_CTL_MC__STOP;
}

// Timer A0_0 interrupt service routine
void TA0_0_IRQHandler(void) {
    // Clear the compare interrupt flag
    TIMER_A0->CCTL[0] &= ~TIMER_A_CCTLN_CCIFG;
	Event_Send( TA0_0_IRQn, 2 );
}


void	ADC_start(void)
{
    // Initialize the shared reference module
    // By default, REFMSTR=1 => REFCTL is used to configure the internal reference
    while(REF_A->CTL0 & REF_A_CTL0_GENBUSY);// If ref generator busy, WAIT
    REF_A->CTL0 |= REF_A_CTL0_VSEL_3 |      // Enable internal 2.5V reference
            REF_A_CTL0_ON;                  // Turn reference on
    REF_A->CTL0 &= ~REF_A_CTL0_TCOFF;       // Enable temperature sensor

    // Configure ADC - Pulse sample mode; ADC14_CTL0_SC trigger
    ADC14->CTL0 |= ADC14_CTL0_SHT0_7 |      // ADC ON, sample period 7
            ADC14_CTL0_ON |
            ADC14_CTL0_SHP;
    // Enable 1/2 AVCC, 14bits
    ADC14->CTL1 |= ADC14_CTL1_BATMAP | ADC14_CTL1_RES_3;
    ADC14->MCTL[0] = ADC14_MCTLN_VRSEL_1 |  // ADC input ch A23 => AVCC sense
            ADC14_MCTLN_INCH_23;
    ADC14->IER0 = 0x0001;                   // ADC_IFG upon conv result-ADCMEM0
    ADC14->CTL0 |= ADC14_CTL0_ENC;
    // Wait for reference generator to settle
    while(!(REF_A->CTL0 & REF_A_CTL0_GENRDY));
    ADC14->CTL0 |= ADC14_CTL0_SC;
}
void	ADC_stop()
{
	// stop REF and ADC14
    ADC14->CTL0 &=  ~(ADC14_CTL0_ON | ADC14_CTL0_ENC);
    REF_A->CTL0 &= ~REF_A_CTL0_ON;                  // Turn reference on
}

// ADC14 interrupt service routine
void ADC14_IRQHandler(void)
{
    switch( ADC14->IV )
    {
    case	0x0C:
        ADC_data = ADC14->MEM[0];
        Event_Send( ADC14_IRQn, 2 );
        break;
    default:
    	break;
    }
}
